@@ -54,7 +54,7 @@
 
 use std::{convert::TryInto, ffi::CString};
 
-use crate::{KcapiError, KcapiResult};
+use crate::{skcipher::AES_BLOCKSIZE, KcapiError, KcapiResult, VMSplice};
 
 ///
 /// # The `KcapiAEADData` Type
@@ -365,7 +365,7 @@ impl KcapiAEAD {
             let ret = kcapi_sys::kcapi_aead_init(&mut handle as *mut _, alg.as_ptr(), flags);
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret.into(),
+                    code: ret,
                     message: format!(
                         "Failed to initialize AEAD handle for algorithm '{}'",
                         algorithm,
@@ -378,7 +378,7 @@ impl KcapiAEAD {
                 .expect("Failed to convert u32 into usize");
             if max_tagsize == 0 {
                 return Err(KcapiError {
-                    code: -libc::EINVAL as i64,
+                    code: -libc::EINVAL,
                     message: format!(
                         "Failed to obtain max authsize for algorithm '{}'",
                         algorithm,
@@ -391,7 +391,7 @@ impl KcapiAEAD {
                 .expect("Failed to convert u32 into usize");
             if blocksize == 0 {
                 return Err(KcapiError {
-                    code: -libc::EINVAL as i64,
+                    code: -libc::EINVAL,
                     message: format!("Failed to obtain blocksize for algorithm '{}'", algorithm,),
                 });
             }
@@ -447,7 +447,7 @@ impl KcapiAEAD {
             let ret = kcapi_sys::kcapi_aead_setkey(self.handle, key.as_ptr(), key.len() as u32);
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret.into(),
+                    code: ret,
                     message: format!("Failed to set key for algorithm '{}'", self.algorithm,),
                 });
             }
@@ -459,7 +459,7 @@ impl KcapiAEAD {
     fn set_inbufsize(&mut self, inlen: usize) -> KcapiResult<()> {
         if self.data.taglen() == 0 {
             return Err(KcapiError {
-                code: -libc::EINVAL as i64,
+                code: -libc::EINVAL,
                 message: format!("Tag size not set for algorithm '{}'", self.algorithm,),
             });
         }
@@ -484,7 +484,7 @@ impl KcapiAEAD {
     fn set_outbufsize(&mut self, outlen: usize) -> KcapiResult<()> {
         if self.data.taglen() == 0 {
             return Err(KcapiError {
-                code: -libc::EINVAL as i64,
+                code: -libc::EINVAL,
                 message: format!("Tag size not set for algorithm '{}'", self.algorithm,),
             });
         }
@@ -537,7 +537,7 @@ impl KcapiAEAD {
     pub fn set_tag(&mut self, tag: Vec<u8>) -> KcapiResult<()> {
         if tag.len() > self.max_tagsize {
             return Err(KcapiError {
-                code: -libc::EINVAL as i64,
+                code: -libc::EINVAL,
                 message: format!(
                     "Invalid tagsize {} > {} for algorithm '{}'",
                     tag.len(),
@@ -551,7 +551,7 @@ impl KcapiAEAD {
             let ret = kcapi_sys::kcapi_aead_settaglen(self.handle, tag.len() as u32);
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret.into(),
+                    code: ret,
                     message: format!(
                         "Failed to set tag length for algorithm '{}'",
                         self.algorithm,
@@ -593,7 +593,7 @@ impl KcapiAEAD {
     pub fn set_tagsize(&mut self, tagsize: usize) -> KcapiResult<()> {
         if tagsize > self.max_tagsize {
             return Err(KcapiError {
-                code: -libc::EINVAL as i64,
+                code: -libc::EINVAL,
                 message: format!(
                     "Invalid tagsize {} > {} for algorithm '{}'",
                     tagsize, self.max_tagsize, self.algorithm,
@@ -605,7 +605,7 @@ impl KcapiAEAD {
             let ret = kcapi_sys::kcapi_aead_settaglen(self.handle, tagsize as u32);
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret.into(),
+                    code: ret,
                     message: format!(
                         "Failed to set tag length for algorithm '{}'",
                         self.algorithm,
@@ -648,7 +648,7 @@ impl KcapiAEAD {
     fn check_aead_input(&self, iv: &[u8]) -> KcapiResult<()> {
         if self.key.is_empty() {
             return Err(KcapiError {
-                code: -libc::EINVAL as i64,
+                code: -libc::EINVAL,
                 message: format!(
                     "Authenticated Encryption key is not set for algorithm '{}'",
                     self.algorithm,
@@ -657,7 +657,7 @@ impl KcapiAEAD {
         }
         if self.data.taglen() == 0 {
             return Err(KcapiError {
-                code: -libc::EINVAL as i64,
+                code: -libc::EINVAL,
                 message: format!(
                     "Tag or Tag size is not set for algorithm '{}'",
                     self.algorithm,
@@ -666,7 +666,7 @@ impl KcapiAEAD {
         }
         if iv.len() != self.ivsize {
             return Err(KcapiError {
-                code: -libc::EINVAL as i64,
+                code: -libc::EINVAL,
                 message: format!(
                     "Invalid IV of size {}, IV must be of size {} for algorithm '{}'",
                     iv.len(),
@@ -746,7 +746,7 @@ impl KcapiAEAD {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret,
+                    code: ret.try_into().expect("failed to convert i64 into i32"),
                     message: format!(
                         "Authenticated Encryption failed for algorithm '{}'",
                         self.algorithm,
@@ -849,7 +849,7 @@ impl KcapiAEAD {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret,
+                    code: ret.try_into().expect("failed to convert i64 into i32"),
                     message: format!(
                         "Authenticated decryption failed for algorithm '{}'",
                         self.algorithm,
@@ -877,6 +877,101 @@ impl Drop for KcapiAEAD {
     }
 }
 
+impl VMSplice for KcapiAEAD {
+    ///
+    /// ## Get Maximum buffer size of VMSPLICE Access
+    ///
+    /// This function returns the maximum number of bytes that can be handled
+    /// by a VMSPLICE call to the kernel.
+    ///
+    fn get_max_splicesize(&self) -> usize {
+        let size: usize;
+        unsafe {
+            size = kcapi_sys::kcapi_get_maxsplicesize(self.handle) as usize;
+        }
+        size
+    }
+
+    ///
+    /// ## Set Maximum Buffer Size for VMSPLICE Access
+    ///
+    /// When using vmsplice/splice to avoid copying of data into the kernel, the
+    /// kernel enforces a maximum number of bytes which can be spliced. If larger
+    /// data is to be processed, sendmsg will be used.
+    ///
+    /// Using this call, the buffer size can be increased.
+    ///
+    /// *NOTE:* Splice uses a pipe pair. Therefore, the maximum number of bytes
+    /// that can be stored with the pipe governs the maximum data size to be
+    /// spliced. Increasing the pipe buffer size is only allowed up to the maximum
+    /// specified with `/proc/sys/fs/pipe-max-size`.
+    ///
+    /// This function takes:
+    /// `size` - A `usize` denoting the size of the vmsplice buffer.
+    ///
+    /// On failure a `KcapiResult` is returned.
+    ///
+    fn set_max_splicesize(&self, size: usize) -> KcapiResult<()> {
+        unsafe {
+            let ret =
+                kcapi_sys::kcapi_set_maxsplicesize(self.handle, size as ::std::os::raw::c_uint);
+
+            if ret < 0 {
+                return Err(KcapiError {
+                    code: ret,
+                    message: format!(
+                        "Unable to set max splice size {} for algorithm {}",
+                        size, self.algorithm,
+                    ),
+                });
+            }
+        }
+        Ok(())
+    }
+}
+
+///
+/// ## Convert CCM nonce into IV
+///
+/// This service function converts a CCM nonce value into an IV usable by the
+/// kernel crypto API.
+///
+/// This function takes:
+/// * nonce - A `Vec<u8>` containing the nonce with length `<= (AES_BLOCKSIZE - 2)`.
+///
+/// On success, returns a `Vec<u8>` containing the new IV.
+/// On failure, returns a `KcapiError`
+///
+/// ## Examples
+///
+/// ```
+/// let nonce = vec![0x41u8; 10];
+/// let iv = kcapi::aead::ccm_nonce_to_iv(nonce)
+///     .expect("Failed to convert AES-CCM nonce to IV");
+/// ```
+///
+pub fn ccm_nonce_to_iv(nonce: Vec<u8>) -> KcapiResult<Vec<u8>> {
+    if nonce.len() > (AES_BLOCKSIZE - 2) {
+        return Err(KcapiError {
+            code: -libc::EINVAL,
+            message: format!(
+                "Cannot pad nonce of length {} > {}",
+                nonce.len(),
+                (AES_BLOCKSIZE - 2),
+            ),
+        });
+    }
+    let padlen: u8 = (AES_BLOCKSIZE - 2 - nonce.len())
+        .try_into()
+        .expect("Failed to convert usize into u8");
+
+    let mut iv = vec![0u8; AES_BLOCKSIZE];
+    iv[1..1 + nonce.len()].clone_from_slice(nonce.as_slice());
+    iv[0] = padlen;
+
+    Ok(iv)
+}
+
 ///
 /// ## Convenience Function for AEAD Encryption (synchronous one-shot)
 ///
