@@ -64,7 +64,7 @@
 
 use std::{convert::TryInto, ffi::CString};
 
-use crate::{KcapiError, KcapiResult, ACCESS_HEURISTIC, INIT_AIO};
+use crate::{KcapiError, KcapiResult, VMSplice, ACCESS_HEURISTIC, INIT_AIO};
 
 ///
 /// # The `KcapiAKCipher` Type
@@ -119,7 +119,7 @@ impl KcapiAKCipher {
             let ret = kcapi_sys::kcapi_akcipher_init(&mut handle as *mut _, alg.as_ptr(), flags);
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret.into(),
+                    code: ret,
                     message: format!(
                         "Failed to initialize akcipher handle for algorithm '{}'",
                         algorithm
@@ -173,7 +173,7 @@ impl KcapiAKCipher {
 
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret.into(),
+                    code: ret,
                     message: format!(
                         "Failed to set private key for algorithm '{}'",
                         self.algorithm
@@ -216,7 +216,7 @@ impl KcapiAKCipher {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret.into(),
+                    code: ret,
                     message: format!(
                         "Failed to set public key for algorithm '{}'",
                         self.algorithm
@@ -262,7 +262,7 @@ impl KcapiAKCipher {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret,
+                    code: ret.try_into().expect("failed to convert i64 into i32"),
                     message: format!("Failed to encrypt for algorithm '{}'", self.algorithm),
                 });
             }
@@ -303,7 +303,7 @@ impl KcapiAKCipher {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret,
+                    code: ret.try_into().expect("failed to convert i64 into i32"),
                     message: format!("Failed to decrypt for algorithm '{}'", self.algorithm),
                 });
             }
@@ -342,7 +342,7 @@ impl KcapiAKCipher {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret,
+                    code: ret.try_into().expect("failed to convert i64 into i32"),
                     message: format!("Failed to sign for algorithm '{}'", self.algorithm),
                 });
             }
@@ -386,7 +386,7 @@ impl KcapiAKCipher {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: -libc::EBADMSG as i64,
+                    code: -libc::EBADMSG,
                     message: format!(
                         "Failed to verify signature for algorithm '{}'",
                         self.algorithm
@@ -406,10 +406,63 @@ impl Drop for KcapiAKCipher {
     }
 }
 
+impl VMSplice for KcapiAKCipher {
+    ///
+    /// ## Get Maximum buffer size of VMSPLICE Access
+    ///
+    /// This function returns the maximum number of bytes that can be handled
+    /// by a VMSPLICE call to the kernel.
+    ///
+    fn get_max_splicesize(&self) -> usize {
+        let size: usize;
+        unsafe {
+            size = kcapi_sys::kcapi_get_maxsplicesize(self.handle) as usize;
+        }
+        size
+    }
+
+    ///
+    /// ## Set Maximum Buffer Size for VMSPLICE Access
+    ///
+    /// When using vmsplice/splice to avoid copying of data into the kernel, the
+    /// kernel enforces a maximum number of bytes which can be spliced. If larger
+    /// data is to be processed, sendmsg will be used.
+    ///
+    /// Using this call, the buffer size can be increased.
+    ///
+    /// *NOTE:* Splice uses a pipe pair. Therefore, the maximum number of bytes
+    /// that can be stored with the pipe governs the maximum data size to be
+    /// spliced. Increasing the pipe buffer size is only allowed up to the maximum
+    /// specified with `/proc/sys/fs/pipe-max-size`.
+    ///
+    /// This function takes:
+    /// `size` - A `usize` denoting the size of the vmsplice buffer.
+    ///
+    /// On failure a `KcapiResult` is returned.
+    ///
+    fn set_max_splicesize(&self, size: usize) -> KcapiResult<()> {
+        unsafe {
+            let ret =
+                kcapi_sys::kcapi_set_maxsplicesize(self.handle, size as ::std::os::raw::c_uint);
+
+            if ret < 0 {
+                return Err(KcapiError {
+                    code: ret,
+                    message: format!(
+                        "Unable to set max splice size {} for algorithm {}",
+                        size, self.algorithm,
+                    ),
+                });
+            }
+        }
+        Ok(())
+    }
+}
+
 fn check_input(handle: &KcapiAKCipher, inp: Vec<u8>) -> KcapiResult<()> {
     if handle.privkey.is_empty() && handle.pubkey.is_empty() {
         return Err(KcapiError {
-            code: -libc::EINVAL as i64,
+            code: -libc::EINVAL,
             message: format!(
                 "Required asymmetric key is not set for algorithm '{}'",
                 handle.algorithm
@@ -418,7 +471,7 @@ fn check_input(handle: &KcapiAKCipher, inp: Vec<u8>) -> KcapiResult<()> {
     }
     if inp.len() > handle.modsize {
         return Err(KcapiError {
-            code: -libc::EINVAL as i64,
+            code: -libc::EINVAL,
             message: format!(
                 "Input to asymmetric cipher is larger than modulus size for algorithm {}",
                 handle.algorithm
