@@ -50,7 +50,7 @@
 //! the initialization, and setkey functions for Counter Mode, Feedback Mode,
 //! and Double Pipeline Mode KDFs.
 //!
-use std::ffi::CString;
+use std::{convert::TryInto, ffi::CString};
 
 use crate::{KcapiError, KcapiResult, INIT_AIO};
 
@@ -120,7 +120,7 @@ impl KcapiKDF {
             let ret = kcapi_sys::kcapi_md_init(&mut handle as *mut _, alg.as_ptr(), !INIT_AIO);
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret.into(),
+                    code: ret,
                     message: format!(
                         "Failed to Initialize hash handle for algorithm '{}'",
                         algorithm,
@@ -131,7 +131,7 @@ impl KcapiKDF {
             digestsize = kcapi_sys::kcapi_md_digestsize(handle) as usize;
             if digestsize == 0 {
                 return Err(KcapiError {
-                    code: -libc::EINVAL as i64,
+                    code: -libc::EINVAL,
                     message: format!("Failed to obtain digestsize for algorithm '{}'", algorithm,),
                 });
             }
@@ -177,7 +177,7 @@ impl KcapiKDF {
             let ret = kcapi_sys::kcapi_md_setkey(self.handle, key.as_ptr(), key.len() as u32);
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret.into(),
+                    code: ret,
                     message: format!("Failed to set key for KDF algorithm '{}'", self.algorithm,),
                 });
             }
@@ -236,7 +236,7 @@ impl KcapiKDF {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret,
+                    code: ret.try_into().expect("failed to convert i64 into i32"),
                     message: format!(
                         "Failed to generate key for KDF algorithm '{}'",
                         self.algorithm,
@@ -291,7 +291,7 @@ impl KcapiKDF {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret,
+                    code: ret.try_into().expect("failed to convert i64 into i32"),
                     message: format!(
                         "Failed to generate key for KDF algorithm '{}'",
                         self.algorithm,
@@ -344,7 +344,7 @@ impl KcapiKDF {
     pub fn fb_kdf(&self, input: Vec<u8>, outsize: usize) -> KcapiResult<Vec<u8>> {
         if input.len() < self.digestsize {
             return Err(KcapiError {
-                code: -libc::EINVAL as i64,
+                code: -libc::EINVAL,
                 message: format!(
                     "Invalid input of length {} < {} for FB-KDF algorithm '{}'",
                     input.len(),
@@ -364,7 +364,7 @@ impl KcapiKDF {
             );
             if ret < 0 {
                 return Err(KcapiError {
-                    code: ret,
+                    code: ret.try_into().expect("failed to convert i64 into i32"),
                     message: format!(
                         "Failed to generate key for KDF algorithm '{}'",
                         self.algorithm,
@@ -423,7 +423,7 @@ pub fn hkdf(
     let mut out = vec![0u8; outsize];
     if ikm.is_empty() {
         return Err(KcapiError {
-            code: -libc::EINVAL as i64,
+            code: -libc::EINVAL,
             message: format!(
                 "Input key material is a required arguement for algorithm '{}'",
                 hashname,
@@ -446,7 +446,7 @@ pub fn hkdf(
         );
         if ret < 0 {
             return Err(KcapiError {
-                code: ret,
+                code: ret.try_into().expect("failed to convert i64 into i32"),
                 message: format!("Failed HKDF operation for algorithm '{}'", hashname,),
             });
         }
@@ -491,7 +491,7 @@ pub fn pbkdf(
     let mut out = vec![0u8; outsize];
     if password.is_empty() || salt.is_empty() {
         return Err(KcapiError {
-            code: -libc::EINVAL as i64,
+            code: -libc::EINVAL,
             message: format!("Invalid input to PBKDF algorithm '{}'", hashname),
         });
     }
@@ -501,7 +501,7 @@ pub fn pbkdf(
         let iter = kcapi_sys::kcapi_pbkdf_iteration_count(hash.as_ptr(), 0);
         if iterations == 0 {
             return Err(KcapiError {
-                code: -libc::EINVAL as i64,
+                code: -libc::EINVAL,
                 message: format!(
                     "Insufficient iteration count {}. Recommended count is {} for '{}'",
                     iterations, iter, hashname,
@@ -521,7 +521,7 @@ pub fn pbkdf(
         );
         if ret < 0 {
             return Err(KcapiError {
-                code: ret,
+                code: ret.try_into().expect("failed to convert i64 into i32"),
                 message: format!("Failed PBKDF operation for algorithm '{}'", hashname,),
             });
         }
